
                  include     "GameEquates.asm"

                  org   32768

map_editor:       jp    map_start   ; normal entry address for editor

                  jp    map_clear   ; clears the map area and compresses before exiting


map_clear:        ld    hl,49152
                  ld    de,49153
                  ld    bc,16383
                  ld    (hl),l
                  ldir

                  ld    hl,#C000
                  ld    de,#FF00
clear_1:          ld    a,2
                  ld    (hl),a
                  ld    (de),a
                  inc   hl
                  inc   de

                  ld    a,l
                  cp    #E9         ; stop at 49385; 49384 = X value on right edge of city
                  jr    c,clear_1

                  ld    hl,#C100
                  ld    de,#C1E8
clear_2:          ld    a,2
                  ld    (hl),a
                  ld    (de),a
                  inc   h
                  inc   d

                  ld    a,h
                  cp    #FF         ; stop at base of map
                  jr    c,clear_2

                ; now build robot enclosure with gates
                  ld    de,55416          ; top of enclosure
                  call  Horz_Enclosure
                  ld    de,57208          ; bottom of enclosure
                  call  Horz_Enclosure

                  ld    hl,55672
                  ld    de,55687
                  ld    a,2
                  ld    b,6
set_gate_sides:   ld    (hl),a
                  ld    (de),a
                  inc   h
                  inc   d
                  djnz  set_gate_sides

                  jp    ExitBASIC   ; exit, compressing map and returning length

Horz_Enclosure:   ld    hl,enclosure_top
                  ld    bc,16
                  ldir
                  ret

enclosure_top:    db    2,2,2,2,2
                  db    3
                  db    9,9,9,9
                  db    3
                  db    2,2,2,2,2



map_start:        call  DecompressMap

                  call  BuildObjectsTable ; gets number of objects in map

                  call  3435
                  ld    a,2
                  call  #1601

                  ld    a,1
                  ld    (redrawmap),a

                  ld    hl,63616
                  ld    (cursor),hl

editloop:         ld    a,(redrawmap)
                  or    a
                  jr    z,noredraw
                  xor   a
                  ld    (redrawmap),a

                  ld    hl,(cursor)
                  ld    de,256*7+15
                  or    a
                  sbc   hl,de
                  ld    de,toptxt
                  call  WordToString

                  ld    hl,(cursor)
                  ld    de,cursortxt
                  call  WordToString

                  ld    a,(objects_in_map)
                  ld    de,numobjectstxt
                  call  ByteToString

                  ld    de,maptext
                  call  PrintText

                  call  DrawMapScreen

                  ld    hl,22528+(32*7)+15
                  ld    (hl),128+16+6

noredraw:
                  ld    a,#fb
                  in    a,(254)
                  rra
                  ld    bc,-256
                  call  nc,MoveCursor

                  ld    a,#fd
                  in    a,(254)
                  rra
                  ld    bc,256
                  call  nc,MoveCursor

                  ld    a,#df
                  in    a,(254)
                  and   2
                  ld    bc,-1
                  call  z,MoveCursor

                  ld    a,#df
                  in    a,(254)
                  rra
                  ld    bc,1
                  call  nc,MoveCursor

                  call  CheckBlockKeys

                  ld    a,#7f             ; B
                  in    a,(254)
                  and   16
                  call  z,Handle_Special_Blocks

                  ld    a,#fd             ; S
                  in    a,(254)
                  and   2
                  call  z,ToggleSolidBlock

                  ld    c,OBJECT_ENERGY
                  ld    a,#ef
                  in    a,(254)
                  rra
                  call  nc,ToggleObject

                  ld    c,OBJECT_HYDROGEN
                  ld    a,#bf
                  in    a,(254)
                  and   16
                  call  z,ToggleObject

                  ld    c,OBJECT_TIME
                  ld    a,#fb
                  in    a,(254)
                  and   16                ; T for time object
                  call  z,ToggleObject

                  ld    a,#fe
                  in    a,(254)
                  and   4
                  jr    z,ExitBASIC       ; X exits to BASIC

                  jp    editloop

ExitBASIC:        call  CompressMap       ; BC = size of map data area at 25088
                  ret                     ; exit to BASIC

Handle_Special_Blocks:
                  ld    a,#7f
                  in    a,(254)
                  and   16
                  jr    z,Handle_Special_Blocks

HSB_Loop:         ld    a,#bf
                  in    a,(254)
                  rra
                  ret   nc                ; Enter quits

                  ld    a,#df             ; P
                  in    a,(254)
                  rra
                  jr    nc,Set_Push_Block

                  jr    HSB_Loop

Set_Push_Block:   ld    hl,(cursor)
                  ld    a,(hl)
                  and   7
                  ret   z                 ; block must have a valid height

                  ld    a,(hl)
                  and   %00101000
                  ret   nz                ; block must not be a collectable object or a non-solid block

                  ld    a,(hl)
                  or    SPECIAL_PUSH
                  ld    (hl),a

                  ld    a,1
                  ld    (redrawmap),a
                  ret


MoveCursor:       ld    hl,(cursor)
                  add   hl,bc
                  ld    a,h
                  cp    192
                  ret   c
                  ld    (cursor),hl
                  ld    a,1
                  ld    (redrawmap),a
                  ret

ToggleObject:     ld    a,#ef
                  in    a,(254)
                  rra
                  jr    nc,ToggleObject         ; wait for release of '0' key
                  ld    a,#bf
                  in    a,(254)
                  and   16
                  jr    z,ToggleObject          ; wait for release of 'H' key
                  ld    a,#fb
                  in    a,(254)
                  and   16
                  jr    z,ToggleObject          ; wait for release of 'T' key

                  ld    hl,(cursor)
                  ld    a,(hl)
                  and   7
                  ret   nz                      ; objects to collect can only exist alone here

                  ld    a,1
                  ld    (redrawmap),a

                  bit   5,(hl)
                  jr    nz,Map_RemObject

Map_AddObject:    ld    a,(objects_in_map)
                  cp    MAX_OBJECTS
                  ret   z
                  inc   a
                  ld    (objects_in_map),a
                  ld    (hl),c
                  ret

Map_RemObject:    ld    (hl),0
                  ld    a,(objects_in_map)
                  dec   a
                  ld    (objects_in_map),a
                  ret

ToggleSolidBlock: ld    a,#fd
                  in    a,(254)
                  and   2
                  jr    z,ToggleSolidBlock      ; wait for release of S key

                  ld    hl,(cursor)
                  ld    a,(hl)
                  and   7
                  ret   z

                  ld    a,(hl)
                  xor   8
                  ld    (hl),a
                  ld    a,1
                  ld    (redrawmap),a
                  ret

CheckBlockKeys:   ld    c,0
                  ld    a,#7f
                  in    a,(254)
                  rra
                  jr    nc,SetBlockVal

                  ld    c,1
                  ld    a,#f7       ; 1 to 5
                  in    a,(254)
                  rra
                  jr    nc,SetBlockVal
                  inc   c
                  rra
                  jr    nc,SetBlockVal
                  inc   c
                  rra
                  jr    nc,SetBlockVal
                  inc   c
                  rra
                  jr    nc,SetBlockVal
                  inc   c
                  rra
                  jr    nc,SetBlockVal

                  inc   c
                  ld    a,#ef       ; 6 to 7
                  in    a,(254)
                  bit   4,a
                  jr    z,SetBlockVal
                  inc   c
                  bit   3,a
                  jr    z,SetBlockVal
                  ret

SetBlockVal:      ld    hl,(cursor)
                  bit   5,(hl)
                  call  nz,Map_RemObject  ; remove any object that may be here

                  ld    (hl),c
                  ld    a,1
                  ld    (redrawmap),a
                  ret

DrawMapScreen:    ld    hl,(cursor)
                  ld    de,256*7+15
                  or    a
                  sbc   hl,de
                  ex    de,hl

                  ld    hl,16384
                  ld    b,15
DMS_1:            push  bc
                  push  de
                  push  hl

                  ld    b,32
DMS_2:            ld    a,(de)

                  call  DrawMapBlock

                  inc   de
                  inc   l
                  djnz  DMS_2

                  pop   hl
                  call  chardown

                  pop   de
                  inc   d

                  pop   bc
                  djnz  DMS_1
                  ret

tempblocktype:    db    0

DrawMapBlock:     push  bc
                  push  de
                  push  hl

                  ex    de,hl

                  ld    (tempblocktype),a

                  bit   5,a
                  jr    z,dms_notobject

                  cp    OBJECT_ENERGY
                  ld    c,"o"
                  jr    z,dms_gotobjchar

                  cp    OBJECT_HYDROGEN
                  ld    c,"h"
                  jr    z,dms_gotobjchar

                  cp    OBJECT_TIME
                  ld    c,"t"
                  jr    z,dms_gotobjchar

                  jr    DMS_Blank

dms_gotobjchar:   ld    a,c
                  jr    drawmapchar

dms_notobject:    and   7
                  jr    z,DMS_Blank

                  add   a,"0"
                  jr    drawmapchar

DMS_Blank:        ld    a," "

drawmapchar:      ld    h,0
                  ld    l,a
                  add   hl,hl
                  add   hl,hl
                  add   hl,hl
                  ld    bc,15360
                  add   hl,bc
                  ex    de,hl

                  ld    b,8
drawmapnum:       ld    a,(de)
                  ld    (hl),a
                  inc   de
                  inc   h
                  djnz  drawmapnum

                  ld    c,56
                  ld    a,(tempblocktype)
                  bit   3,a
                  jr    z,dmc_colour
                  ld    c,%00110000

dmc_colour:       dec   h
                  call  Scr2Attr

                  ld    a,(tempblocktype)
;                  bit   5,a
                  and   %00110000         ; flash collectable objects and special block types
                  jr    z,dmc_notobj
                  set   7,c

dmc_notobj:       ld    (hl),c      ; set block colour

                  pop   hl
                  pop   de
                  pop   bc
                  ret


PrintText:        ld    a,(de)
                  cp    255
                  ret   z
                  rst   16
                  inc   de
                  jr    PrintText

BuildObjectsTable:
                  ld    hl,Objects_Table
                  ld    de,49152
                  ld    b,0
BOT_1:            ld    a,(de)
                  bit   5,a
                  jr    z,BOT_Next

                  ld    (hl),e
                  inc   hl
                  ld    (hl),d
                  inc   hl
                  ld    (hl),a
                  inc   hl

                  inc   b     ; increment total objects count
                  ld    a,b
                  cp    MAX_OBJECTS
                  jr    z,BOT_Exit

BOT_Next:         inc   de
                  ld    a,d
                  or    e
                  jr    nz,BOT_1

BOT_Exit:         ld    a,b
                  ld    (objects_in_map),a
                  ret


                  include     "MapCodec.asm"
                  include     "Text.asm"
                  include     "Misc.asm"

maptext:
                  db    22,16,0
                  db    "TOP: "
toptxt:           db    "00000"
                  db    22,16,16
                  db    "CURSOR: "
cursortxt:        db    "00000"
                  db    22,17,0
                  db    "OBJECTS: "
numobjectstxt:    db    "000"
                  db    255

cursor:           dw    0
redrawmap:        db    0

objects_in_map:   db    0     ; objects found in map

Objects_Table:    ds    MAX_OBJECTS*3


