
SetupScrTable:    ld    ix,scrtable
                  ld    hl,16384
                  ld    b,192
setst1:           ld    (ix+0),l
                  ld    (ix+1),h
                  call  pixeldown
                  inc   ix
                  inc   ix
                  djnz  setst1
                  ret

scrtable:         ds    192*2


; entry H = pixels down, L = chars across
; exit  HL = display address
MapScreenAddress: push  de
                  ld    a,l
                  ld    l,h
                  ld    h,0
                  add   hl,hl
                  ld    de,scrtable
                  add   hl,de
                  add   a,(hl)
                  inc   hl
                  ld    h,(hl)
                  ld    l,a
                  pop   de
                  ret

; entry HL = display address
; exit  HL = attr address
Scr2Attr:         ld    a,h
                  rrca
                  rrca
                  rrca
                  and   3
                  or    &58
                  ld    h,a
                  ret

pixelup:          ld    a,h
                  dec   h
                  and   7
                  ret   nz
                  ld    a,8
                  add   a,h
                  ld    h,a
charup:           ld    a,l
                  sub   32
                  ld    l,a
                  ret   nc
                  ld    a,h
                  sub   8
                  ld    h,a
                  cp    HIGH BACKBUFFER   ; C set = off screen
                  ret

pixelup_de:       ld    a,d
                  dec   d
                  and   7
                  ret   nz
                  ld    a,8
                  add   a,d
                  ld    d,a
charup_de:        ld    a,e
                  sub   32
                  ld    e,a
                  ret   nc
                  ld    a,d
                  sub   8
                  ld    d,a
                  cp    HIGH BACKBUFFER   ; C set = off screen
                  ret


pixeldown:        inc   h
                  ld    a,h
                  and   7
                  ret   nz
                  ld    a,h
                  sub   8
                  ld    h,a
chardown:         ld    a,l
                  add   a,32
                  ld    l,a
                  and   224
                  ret   nz
                  ld    a,h
                  add   a,8
                  ld    h,a
                  ret

pixeldown_de:     inc   d
                  ld    a,d
                  and   7
                  ret   nz
                  ld    a,d
                  sub   8
                  ld    d,a
chardown_de:      ld    a,e
                  add   a,32
                  ld    e,a
                  and   224
                  ret   nz
                  ld    a,d
                  add   a,8
                  ld    d,a
                  ret



; HL = top left display address
; B = width
; C = height
; A = colour

Draw_Box:         ld    (box_colour+1),a

                  push  bc
                  push  hl
                  ld    de,Box_UpperLeft
                  call  Draw_Box_Char

                  call  Draw_Box_Horz

                  ld    de,Box_UpperRight
                  call  Draw_Box_Char
                  pop   hl
                  pop   bc

                  push  bc
                  push  hl
                  dec   c
DB_FindBase:      call  chardown
                  dec   c
                  jr    nz,DB_FindBase

                  ld    de,Box_LowerLeft
                  call  Draw_Box_Char

                  call  Draw_Box_Horz

                  ld    de,Box_LowerRight
                  call  Draw_Box_Char
                  pop   hl
                  pop   bc

                  push  bc
                  push  hl
                  call  Draw_Box_Vert
                  pop   hl
                  pop   bc

                  push  bc
                  push  hl
                  ld    a,l
                  add   a,b
                  dec   a
                  ld    l,a
                  call  Draw_Box_Vert
                  pop   hl
                  pop   bc

                  ret


Draw_Box_Horz:    inc   l
                  dec   b
                  dec   b
DBH_1:            ld    de,Box_Horizontal
                  call  Draw_Box_Char
                  inc   l
                  djnz  DBH_1
                  ret

Draw_Box_Vert:    call  chardown
                  dec   c
                  dec   c
DBV_1:            ld    de,Box_Vertical
                  call  Draw_Box_Char
                  call  chardown
                  dec   c
                  jr    nz,DBV_1
                  ret

Draw_Box_Char:    push  hl
                  ld    a,8
DBC_Loop:         ex    af,af'
                  ld    a,(de)
                  ld    (hl),a
                  inc   de
                  inc   h
                  ex    af,af'
                  dec   a
                  jr    nz,DBC_Loop

                  dec   h
                  call  Scr2Attr
box_colour:       ld    (hl),0
                  pop   hl
                  ret

Box_Horizontal:
                  db    %00000000
                  db    %00111100
                  db    %11100111
                  db    %10111101
                  db    %10111101
                  db    %11100111
                  db    %00111100
                  db    %00000000

Box_Vertical:
                  db    %00111100
                  db    %00100100
                  db    %01111110
                  db    %01011010
                  db    %01011010
                  db    %01111110
                  db    %00100100
                  db    %00111100

Box_UpperLeft:
                  db    %00000000
                  db    %00111000
                  db    %01000111
                  db    %01010101
                  db    %01000101
                  db    %00111011
                  db    %00100100
                  db    %00111100

Box_UpperRight:
                  db    %00000000
                  db    %00011100
                  db    %11100010
                  db    %10101010
                  db    %10100010
                  db    %11011100
                  db    %00100100
                  db    %00111100

Box_LowerLeft:
                  db    %00111100
                  db    %00100100
                  db    %00111011
                  db    %01000101
                  db    %01010101
                  db    %01000111
                  db    %00111000
                  db    %00000000

Box_LowerRight:
                  db    %00111100
                  db    %00100100
                  db    %11011100
                  db    %10100010
                  db    %10101010
                  db    %11100010
                  db    %00011100
                  db    %00000000



