
InitRobots:       ld    ix,Robots
                  ld    hl,InitRobotTable
                  ld    b,NUM_ROBOTS
Init_R1:          push  bc

                  ld    e,(hl)      ; E = X in map
                  inc   hl
                  ld    d,(hl)      ; D = Y in map
                  inc   hl
                  ld    a,(hl)      ; A = path type
                  inc   hl
                  ld    c,(hl)      ; C = speed
                  inc   hl

                  push  hl
                  ld    (ix+OFS_X),e
                  ld    (ix+OFS_Y),d
                  ld    (ix+OFS_PATHTYPE),a
                  ld    (ix+OFS_SPEED),c
                  ld    (ix+OFS_SPEEDCOUNT),c

                  ld    (ix+OFS_HEIGHT),0
                  ld    (ix+OFS_MOVEFLAGS),0

                  call  Init_Robot_Path
                  pop   hl

                  ld    bc,OFS_SIZE
                  add   ix,bc

                  pop   bc
                  djnz  Init_R1
                  ret

DEF_ROBOT         macro location,pathtype,speed
                  dw    location
                  db    pathtype
                  db    speed
                  endm
DEF_ROBOT_SIZE    equ   4     ; size of robot setup entries

InitRobotTable:   DEF_ROBOT   55673, PATH_NORMAL, 2
                  DEF_ROBOT   55674, PATH_NORMAL, 2
                  DEF_ROBOT   55675, PATH_NORMAL, 2
                  DEF_ROBOT   55676, PATH_NORMAL, 2
                  DEF_ROBOT   55677, PATH_NORMAL, 2
                  DEF_ROBOT   55678, PATH_NORMAL, 2
                  DEF_ROBOT   55679, PATH_NORMAL, 2
                  DEF_ROBOT   55680, PATH_NORMAL, 2

                  DEF_ROBOT   55929, PATH_NORMAL, 2
                  DEF_ROBOT   55930, PATH_NORMAL, 2
                  DEF_ROBOT   55931, PATH_NORMAL, 2
                  DEF_ROBOT   55932, PATH_NORMAL, 2

                  DEF_ROBOT   65119, PATH_PERIMETER_L, 1

                  DEF_ROBOT   55310, PATH_PERIMETER_L, 2
                  DEF_ROBOT   55317, PATH_PERIMETER_R, 2

                  DEF_ROBOT   63670, PATH_DEFLECTOR, 1
                  DEF_ROBOT   63676, PATH_DEFLECTOR, 1

NUM_ROBOTS        equ   ($-InitRobotTable)/DEF_ROBOT_SIZE

Hitman_Flag:      db    0

Move_Robots:      xor   a
                  ld    (Hitman_Flag),a

                  ld    ix,Robots
                  ld    b,NUM_ROBOTS

Move_Robot_Loop:  push  bc

                  ld    l,(ix+OFS_X)
                  ld    h,(ix+OFS_Y)
                  ld    c,(ix+OFS_HEIGHT)
                  ld    (temprobotposn),hl

                  ld    de,(Player+OFS_X)

                  ld    a,(Player+OFS_HEIGHT)
                  cp    c
                  jr    nz,HandleThisRobot

                  ld    a,d
                  sub   h
                  jr    z,Test_Hit_Man_1
                  cp    1
                  jr    z,Test_Hit_Man_1
                  cp    255
                  jr    nz,HandleThisRobot

Test_Hit_Man_1:   ld    a,e
                  sub   l
                  jr    z,RobotHasHitMan
                  cp    1
                  jr    z,RobotHasHitMan
                  cp    255
                  jr    nz,HandleThisRobot

RobotHasHitMan:   ld    a,(Hitman_Flag)
                  or    a
                  jp    nz,Next_Robot
                  inc   a
                  ld    (Hitman_Flag),a

                  push  bc
                  push  de
                  push  hl

                  ld    bc,#5010
                  call  WhiteNoise

                  ld    a,10
                  call  Sub_Energy

                  pop   hl
                  pop   de
                  pop   bc
                  jr    Next_Robot

HandleThisRobot:  dec   (ix+OFS_SPEEDCOUNT)
                  jr    nz,Next_Robot

                  ld    a,(ix+OFS_SPEED)
                  ld    (ix+OFS_SPEEDCOUNT),a

                  ld    a,(hl)
                  and   %00111111
                  ld    (hl),a

                  call  Move_Robot_Path

                  ld    hl,(temprobotposn)
                  ld    (ix+OFS_X),l
                  ld    (ix+OFS_Y),h
                  ld    (ix+OFS_HEIGHT),c

                  ld    a,(hl)
                  or    OBJECT_ROBOT
                  ld    (hl),a

Next_Robot:       ld    bc,OFS_SIZE
                  add   ix,bc
                  pop   bc
                  dec   b
                  jp    nz,Move_Robot_Loop
                  ret

; HL = map location to test
; C  = Height
; B  = must be preserved
Can_Move_Robot_Here:
                  ld    a,h
                  cp    high MAP
                  ccf
                  ret   nc

                  call  Test_Move_Robot_Here
                  ret   nc
                  inc   c                 ; and test for height above (robot is 2 chars high)
                  call  Test_Move_Robot_Here
                  dec   c
                  ret   nc

                  ld    (temprobotposn),hl
                  scf
                  ret

; returns with Carry set if we can move to this location
Test_Move_Robot_Here:
                  ld    a,(hl)
                  and   7
                  jp    z,TMRH_NoBlock

                  cp    c
                  ret   z           ; block at this height; C clear

                  ret   c           ; higher than the block height; C set

                  bit   3,(hl)      ; test for solid column of blocks
                  ret   z           ; block is solid, C clear

                  scf
                  ret

TMRH_NoBlock:     ld    a,(hl)
                  and   %11000000
                  scf
                  ret   z
                  or    a
                  ret


temprobotposn:    dw    0


Init_Robot_Path:  ld    de,Init_Robot_Path_Table
                  jr    Go_RobotPathVect

Move_Robot_Path:  ld    de,Move_Robot_Path_Table

Go_RobotPathVect: ld    h,0
                  ld    l,(ix+OFS_PATHTYPE)
                  add   hl,hl
                  add   hl,de
                  ld    e,(hl)
                  inc   hl
                  ld    d,(hl)
                  ex    de,hl
                  jp    (hl)



Init_Robot_Path_Table:
                  dw    Init_PATH_NORMAL
                  dw    Init_PATH_PERIMETER_L
                  dw    Init_PATH_PERIMETER_R
                  dw    Init_PATH_DEFLECTOR

Move_Robot_Path_Table:
                  dw    Move_PATH_NORMAL
                  dw    Move_PATH_PERIMETER_L
                  dw    Move_PATH_PERIMETER_R
                  dw    Move_PATH_DEFLECTOR




Init_PATH_NORMAL: ret

Move_PATH_NORMAL: ld    b,0
                  ld    de,(Player+OFS_X)

                  bit   0,(ix+OFS_MOVEFLAGS)
                  jr    nz,MoveR_X

                  ld    hl,(temprobotposn)
                  ld    a,d
                  sub   h
                  jr    z,MoveR_X

                  jr    nc,MoveR_IncY

MoveR_DecY:       dec   h
                  call  Can_Move_Robot_Here
                  set   7,b
                  jr    c,Move_R_Next
                  res   7,b
                  jr    MoveR_X

MoveR_IncY:       inc   h
                  call  Can_Move_Robot_Here
                  set   7,b
                  jr    c,Move_R_Next
                  res   7,b

MoveR_X:          ld    hl,(temprobotposn)
                  ld    a,e
                  sub   l
                  jr    z,Move_R_Next

                  jr    nc,MoveR_IncX

MoveR_DecX:       dec   l
                  call  Can_Move_Robot_Here
                  set   7,b
                  jr    c,Move_R_Next
                  res   7,b
                  jr    Move_R_Next

MoveR_IncX:       inc   l
                  call  Can_Move_Robot_Here
                  set   7,b
                  jr    c,Move_R_Next
                  res   7,b

Move_R_Next:      bit   7,b
                  ret   nz          ; robot has already moved

                  ; move robot in a random direction
                  ld    hl,(temprobotposn)
                  ld    e,1
                  ld    a,r
                  bit   5,a
                  jr    z,robot_disp
                  ld    e,-1
robot_disp:       rra
                  ld    a,e
                  jr    c,move_robot_LR
                  add   a,h
                  ld    h,a
                  jr    test_robot_rand
move_robot_LR:    add   a,l
                  ld    l,a                  
test_robot_rand:  call  Can_Move_Robot_Here
                  ld    a,(ix+OFS_MOVEFLAGS)
                  xor   1
                  ld    (ix+OFS_MOVEFLAGS),a
                  ret

; OFS_MOVEFLAGS holds current direction for this path
; each incremental value turns left from preceding value
ROBOT_DIR_LEFT    equ   0
ROBOT_DIR_DOWN    equ   1
ROBOT_DIR_RIGHT   equ   2
ROBOT_DIR_UP      equ   3

DISP_LEFT         dw    -1
DISP_DOWN         dw    256
DISP_RIGHT        dw    1
DISP_UP           dw    -256

Init_PATH_PERIMETER_L:
                  ld    (ix+OFS_MOVEFLAGS),ROBOT_DIR_UP
                  ret

; relative to current direction, try to move left first, then forwards, right, and finally a U-turn
Move_PATH_PERIMETER_L:
                  ; first try left
                  ld    a,(ix+OFS_MOVEFLAGS)
                  inc   a     ; move left
                  call  Try_Robot_Dir_Addr
                  ret   c

                  ; then try forwards
                  ld    a,(ix+OFS_MOVEFLAGS)
                  call  Try_Robot_Dir_Addr
                  ret   c

                  ; then try right
                  ld    a,(ix+OFS_MOVEFLAGS)
                  dec   a     ; move right
                  call  Try_Robot_Dir_Addr
                  ret   c

                  ; else U-turn
                  ld    a,(ix+OFS_MOVEFLAGS)
                  add   a,2   ; U-Turn
                  call  Try_Robot_Dir_Addr
                  ret

Init_PATH_PERIMETER_R:
                  ld    (ix+OFS_MOVEFLAGS),ROBOT_DIR_UP
                  ret

; relative to current direction, try to move right first, then forwards, left, and finally a U-turn
Move_PATH_PERIMETER_R:
                  ; first try right
                  ld    a,(ix+OFS_MOVEFLAGS)
                  dec   a     ; move right
                  call  Try_Robot_Dir_Addr
                  ret   c

                  ; then try forwards
                  ld    a,(ix+OFS_MOVEFLAGS)
                  call  Try_Robot_Dir_Addr
                  ret   c

                  ; then try left
                  ld    a,(ix+OFS_MOVEFLAGS)
                  inc   a     ; move left
                  call  Try_Robot_Dir_Addr
                  ret   c

                  ; else U-turn
                  ld    a,(ix+OFS_MOVEFLAGS)
                  add   a,2   ; U-Turn
                  call  Try_Robot_Dir_Addr
                  ret

Init_PATH_DEFLECTOR:
                  ld    (ix+OFS_MOVEFLAGS),ROBOT_DIR_RIGHT
                  ret

; relative to current direction, tries to move forwards; if it cannot then it turns to the right
Move_PATH_DEFLECTOR:
                  ; try to move forwards
                  ld    a,(ix+OFS_MOVEFLAGS)
                  call  Try_Robot_Dir_Addr
                  ret   c

                  ; else turn to the right
                  ld    a,(ix+OFS_MOVEFLAGS)
                  dec   a     ; move right
                  call  Try_Robot_Dir_Addr
                  ret



Try_Robot_Dir_Addr:
                  and   3
                  ld    b,a

                  ld    hl,DISP_LEFT
                  add   a,a
                  ld    d,0
                  ld    e,a
                  add   hl,de
                  ld    e,(hl)
                  inc   hl
                  ld    d,(hl)

                  ld    hl,(temprobotposn)
                  add   hl,de

                  ld    a,(hl)
                  and   %11000000
                  cp    OBJECT_ROBOT
                  scf
                  jr    z,TRDA_Move

                  call  Can_Move_Robot_Here     ; carry set if okay to move and new robot posn stored in (temprobotposn)
                  ret   nc

TRDA_Move:        ld    (temprobotposn),hl
                  ld    (ix+OFS_MOVEFLAGS),b
                  ret

