

GFX               macro gfxword
                  db    high gfxword
                  db    low  gfxword
                  endm

                  include     "GameEquates.asm"

                  org   32768
                  jp    Start

; ensure all of these block graphics appear in a single 256 byte page

singleblock:      GFX      %0000001111000000
                  GFX      %0000110000110000
                  GFX      %0011000110001100
                  GFX      %1100011001100011
                  GFX      %1100011001100011
                  GFX      %1011000110001110
                  GFX      %1101110000111011
                  GFX      %1010101111101110

baseblock:        GFX      %0101010110111011
                  GFX      %1010101011101110
                  GFX      %0101010110111011
                  GFX      %1010101011101110
                  GFX      %0101010110111011
                  GFX      %0010101011101100
                  GFX      %0000010110110000
                  GFX      %0000001011000000

midblock:         GFX      %0101010110111011
                  GFX      %1010101011101110
                  GFX      %0101010110111011
                  GFX      %1010101011101110
                  GFX      %0101010110111011
                  GFX      %1010101011101110
                  GFX      %0101010110111011
                  GFX      %1010101011101110


Start:            di

                  call  SetupScrTable
                  call  Set_Facing_Right

                  ld    a,255
                  ld    (No_Object+OFS_HEIGHT),a

Main_Menu:        ld    a,(view_dir)
                  ld    (old_view_dir),a

                  call  Set_Facing_Right

                  ld    a,1
                  ld    (on_title_screen),a

                  call  InitTitles

                  ld    a,7
                  out   (254),a

                  ld    a,56
                  call  ClearScreen

                  ld    de,menu.txt
                  call  WriteText

Wait_Enter:       call  UpdateTitles

                  ld    a,#bf
                  in    a,(254)
                  rra
                  jr    nc,Game_Start

                  ld    a,#fe       ; CAPS key
                  in    a,(254)
                  rra
                  jr    c,Wait_Enter

                  ld    a,#fb       ; CS + E exits to BASIC
                  in    a,(254)
                  and   4
                  jr    nz,Wait_Enter

                  ld    iy,23610
                  ld    hl,#2758
                  exx
                  ei
                  ret

Game_Start:       xor   a
                  ld    (on_title_screen),a

                  ld    a,(old_view_dir)
                  call  Set_View

                  ld    a,1
                  out   (254),a

                  call  DecompressMap
                  call  CountHydrogen     ; get number of hydrogen bombs in map

                  ld    a,MAX_ENERGY
                  ld    (energy),a
                  ld    a,MAX_TIME
                  ld    (time),a

                  ld    a,%00001001
                  call  ClearScreen

                  ld    hl,16384+1
                  ld    bc,#1E12
                  ld    a,%00001110
                  ld    (displaybordercol),a    ; colour of area surrounding main map display area
                  call  Draw_Box

                  call  Render_HUD
                  call  Draw_View_Dir

;                  ld    ix,22528+(32*8)+15
;                  ld    a,%00010110
;                  ld    (ix+0),a
;                  ld    (ix+1),a
;                  ld    (ix+32),a
;                  ld    (ix+33),a

;                  call  SetObjectsInMap
                  call  InitRobots
                  call  Init_Effects

                  ld    hl,64896
                  ld    (Player+OFS_X),hl
                  ld    a,OBJECT_PLAYER
                  ld    (hl),a
                  xor   a
                  ld    (Player+OFS_HEIGHT),a
                  ld    (Player+OFS_FALLHEIGHT),a

                  xor   a
                  ld    (dofallwobble),a
                  ld    (man_dead),a

                  call  DrawEnergyBar
                  call  DrawTimeBar

start_new_level:  xor   a
                  ld    (Carry_Hydrogen),a
                  ld    (Carry_Block),a

                  ld    a,(Hydrogen_Remaining)
                  ld    hl,0
                  ld    de,20*40     ; ~20fps * 40 seconds/bomb
get_max_time:     add   hl,de
                  dec   a
                  jr    nz,get_max_time

                  ld    l,h
                  ld    h,0
                  ld    (timeword),hl
                  ld    (timewordinit),hl

reinitgameloop:   call  SetPlayAreaAttrs

gameloop:         call  ClearBackBuffer

                  ld    hl,framecount
                  inc   (hl)

                  call  Do_Effects
                  call  Move_Robots
                  call  Move_Player
                  call  Alter_View
                  call  ModDiff

                  ld    hl,(Player+OFS_X)
diff:             ld    de,#02F0

                  ld    a,h
                  sub   d
                  ld    h,a
                  ld    a,l
                  sub   e
                  ld    l,a

                  ex    de,hl
                  call  DrawScreen

                  ld    a,(dofallwobble)
                  or    a
                  call  nz,LargeFallWobble

                  call  CopyBackBuffer

                  ld    a,(man_dead)
                  or    a
                  jp    nz,Game_Over

                  call  Time_Tick

                ; test for leaving city with hydrogen
                  ld    a,(Carry_Hydrogen)
                  or    a
                  jr    z,check_abort_key

                  ld    a,(Player+OFS_HEIGHT)
                  or    a
                  jr    nz,check_abort_key

                  ld    hl,(Player+OFS_X)
                  ld    a,h
                  inc   h
                  jp    z,Hydrogen_Out_Of_City
                  cp    #c0
                  jp    z,Hydrogen_Out_Of_City

                  ld    a,l
                  or    a
                  jr    z,Hydrogen_Out_Of_City
                  cp    #e8
                  jr    z,Hydrogen_Out_Of_City


check_abort_key:  ld    a,#fe       ; X exits
                  in    a,(254)
                  and   4
                  jr    nz,gameloop

                  ld    de,askquit.txt
                  call  WriteText

                  call  Wait_No_Key

wait_Y_N:         ld    a,#df       ; Y
                  in    a,(254)
                  and   16
                  jp    z,Main_Menu

                  ld    a,#7f       ; N
                  in    a,(254)
                  and   8
                  jr    z,reinitgameloop

                  jr    wait_Y_N

Wait_No_Key:      xor   a
                  in    a,(254)
                  and   31
                  cp    31
                  jr    nz,Wait_No_Key
                  ret

askquit.txt:      db    C_PAPER,0,C_INK,5
                  db    C_DOUBLE
                  db    C_AT,8,6
                  db    "do you wish to quit?"
                  db    C_AT,11,14
                  db    "y/n"
                  db    C_SINGLE
                  db    0

Game_Over:
                  ld    a,70
                  call  ClearScreen

                  ld    de,gameover.txt
                  call  WriteText

                  ld    hl,375
                  ld    de,2000
                  call  Beep

                  jp    Main_Menu

Hydrogen_Out_Of_City:
                  ld    hl,20480+(32*5)+24
                  ld    de,Space32.Gfx
                  call  Draw_16x16

                  xor   a
                  ld    (Carry_Hydrogen),a

                  ld    hl,Hydrogen_Remaining
                  dec   (hl)
                  jr    z,game_complete

                  ld    a,(hl)
                  ld    de,bombsleft.txt
                  call  ByteToString2

                  ld    de,getnexthydrogen.txt
                  call  WriteText
                  ld    hl,375
                  ld    de,2000
                  call  Beep

                  call  SetPlayAreaAttrs
                  jp    gameloop


game_complete:    ld    de,gamecomplete.txt
                  call  WriteText
                  ld    hl,475
                  ld    de,3000
                  call  Beep

                  jp    Main_Menu

menu.txt:         db    C_PAPER,7,C_INK,2
                  db    C_DOUBLE
                  db    C_AT,19,6
                  db    "press ENTER to start"
                  db    C_SINGLE
                  db    0

gameover.txt:     db    C_PAPER,0,C_INK,4
                  db    C_DOUBLE
                  db    C_AT,10,11
                  db    "game over"
                  db    C_SINGLE
                  db    0

gamecomplete.txt:
                  db    C_PAPER,2,C_INK,7
                  db    C_SINGLE
                  db    C_AT,5,4,  "                        "
                  db    C_DOUBLE
                  db    C_AT,6,4,  "          YAY!          "
                  db    C_SINGLE
                  db    C_AT,8,4,  "                        "
                  db    C_DOUBLE
                  db    C_AT,9,4,  "        YA CUNT!        "
                  db    C_SINGLE
                  db    C_AT,11,4, "                        "
                  db    0

getnexthydrogen.txt:
                  db    C_PAPER,2,C_INK,7,C_SINGLE
                  db    C_AT,6,3,  "                          "
                  db    C_AT,7,3,  "        well done!        "
                  db    C_AT,8,3,  "                          "
                  db    C_AT,9,3,  "   bombs remaining: "
bombsleft.txt:    db    "00    "
                  db    C_AT,10,3, "                          "
                  db    0

Carry_Block:      db    0
Carry_Hydrogen:   db    0
Hydrogen_Remaining:
                  db    0
displaybordercol: db    0     ; colour of area surrounding main map display area
man_dead:         db    0

SetPlayAreaAttrs: ld    hl,22528+32+2
                  ld    b,16
clrplayattr:      push  bc
                  ld    d,h
                  ld    e,l
                  inc   e
                  ld    bc,27
                  ld    (hl),56
                  ldir
                  inc   hl
                  inc   hl
                  inc   hl
                  inc   hl
                  inc   hl
                  pop   bc
                  djnz  clrplayattr
                  ret


ModDiff:          ld    a,#f7
                  in    a,(254)
                  ld    bc,#00FF
                  rra
                  call  nc,mdiff
                  ld    bc,#0001
                  rra
                  call  nc,mdiff
                  ld    bc,#FF00
                  rra
                  call  nc,mdiff
                  ld    bc,#0100
                  rra
                  call  nc,mdiff
                  ret
mdiff:            push  af
                  ld    hl,(diff+1)
                  ld    a,h
                  add   a,b
                  ld    h,a
                  ld    a,l
                  add   a,c
                  ld    l,a
                  ld    (diff+1),hl
                  pop   af
                  ret

CountHydrogen:    ld    hl,49152
                  ld    c,l         ; C = 0
CntHydro_Loop:    ld    a,(hl)
                  cp    OBJECT_HYDROGEN
                  jr    nz,CntHydro_1

                  inc   c

CntHydro_1:       inc   hl
                  ld    a,h
                  or    l
                  jr    nz,CntHydro_Loop

                  ld    a,c
                  ld    (Hydrogen_Remaining),a
                  ret


Move_Player:      call  Anim_ManEye

                  ld    iy,Player

                  ld    hl,(Player+OFS_X)

                  ld    a,(hl)
                  and   %00111111
                  ld    (hl),a

                  ld    a,(iy+OFS_HEIGHT)
                  or    a
                  jr    z,no_man_fall
                  dec   a
                  ld    c,a
                  ld    a,1
                  ld    (testing_man_drop),a
                  call  Can_Move_Here
                  ld    a,0
                  ld    (testing_man_drop),a
                  jr    nc,no_man_fall

                ; man is falling
                  push  hl
                  ld    a,(Player+OFS_FALLHEIGHT)
                  inc   a
                  ld    (Player+OFS_FALLHEIGHT),a
                  and   7
                  inc   a
                  rrca
                  rrca
                  rrca
                  ld    h,0
                  ld    l,a
                  rl    l
                  rl    h
                  rl    l
                  rl    h
                  ld    de,10
                  call  Beep
                  pop   hl
                  jr    Set_Man_Posn      ; man has fallen one block height

no_man_fall:      ld    a,(Player+OFS_FALLHEIGHT)
                  cp    6
                  jr    c,no_man_fallwobble

                  ld    (dofallwobble),a

no_man_fallwobble:
                  xor   a
                  ld    (Player+OFS_FALLHEIGHT),a

                  ld    a,#fe
                  in    a,(254)
                  and   8     ; C ?
                  call  z,Man_Carry

                  ld    a,#7f
                  in    a,(254)
                  rra
                  call  nc,Man_Jump

                  ld    a,#fb
                  in    a,(254)
                  rra
                  call  nc,Man_Up

                  ld    a,#fd
                  in    a,(254)
                  rra
                  call  nc,Man_Down

                  ld    a,#df
                  in    a,(254)
                  and   2
                  call  z,Man_Left

                  ld    a,#df
                  in    a,(254)
                  rra
                  call  nc,Man_Right


Set_Man_Posn:     ld    hl,(Player+OFS_X)
                  ld    a,(hl)
                  bit   5,a               ; object?
                  jr    z,Set_Man_P1
                  ld    a,(Player+OFS_HEIGHT)
                  or    a
                  jr    nz,Set_Man_P1     ; on ground level?

                  ld    a,(hl)
                  ld    (hl),0            ; remove object

                  cp    OBJECT_ENERGY
                  jr    z,Set_Got_Energy

                  cp    OBJECT_HYDROGEN
                  jr    z,Set_Got_Hydrogen

                  cp    OBJECT_TIME
                  jr    z,Set_Got_Time
                  jr    Set_Man_P1

Set_Got_Hydrogen: push  hl
                  call  Got_Hydrogen_Message
                  pop   hl
                  jr    Set_Man_P1

Set_Got_Time:     push  hl
                  call  Got_Time
                  pop   hl
                  jr    Set_Man_P1

Set_Got_Energy:   push  hl
                  ld    hl,900
                  ld    de,47
                  call  Beep
                  ld    a,20
                  call  Add_Energy
                  pop   hl

Set_Man_P1:       ld    a,(hl)
                  or    OBJECT_PLAYER
                  ld    (hl),a
                  ret

Man_Jump:         ld    hl,(Player+OFS_X)
                  ld    a,(Player+OFS_HEIGHT)
                  cp    1
                  adc   a,0
                  inc   a
                  ld    c,a
                  call  Can_Move_Here
                  ret   nc

                  push  hl
                  push  bc
                  ld    b,3
      MJ_Sound:   push  bc
                  ld    h,b
                  ld    l,0
                  ld    de,25
                  call  Beep
                  pop   bc
                  djnz  MJ_Sound
                  pop   bc
                  pop   hl
                  ret

Man_Carry:        ld    a,(Carry_Block)
                  or    a
                  jr    nz,Man_Dropping

                  ld    hl,(Player+OFS_X)
                  ld    a,(hl)
                  bit   3,a
                  jr    nz,Man_No_Carry_Drop_Sound    ; cannot pick up a non-solid block

                  and   SPECIAL_MASK
                  cp    SPECIAL_PUSH
                  jr    z,Man_No_Carry_Drop_Sound     ; cannot pick up a push block

                  ld    a,(hl)
                  and   7
                  jr    z,Man_No_Carry_Drop_Sound
                  ld    c,a   ; height of the push block
                  inc   c
                  ld    a,(Player+OFS_HEIGHT)
                  cp    c
                  jr    nz,Man_No_Carry_Drop_Sound    ; must be at height of push block + 1 to pick it up; ie: on top of it

                  ld    a,(hl)
                  and   248
                  ld    c,a
                  ld    a,(hl)
                  dec   a
                  and   7
                  or    c
                  ld    (hl),a

                  ld    hl,20480+(32*5)+21
                  ld    de,singleblock    ; draws the singleblock + baseblock gfx = one complete map block (16x16)
                  call  Draw_16x16

                  ld    a,1
                  ld    (Carry_Block),a

                  ld    hl,900
                  ld    de,47
                  call  Beep
                  ret

; dropping a block we're carrying
Man_Dropping:     ld    hl,(Player+OFS_X)
                  ld    a,(hl)
                  bit   3,a
                  jr    nz,Man_No_Carry_Drop_Sound    ; cannot drop onto a non-solid block

                  and   SPECIAL_MASK
                  cp    SPECIAL_PUSH
                  jr    z,Man_No_Carry_Drop_Sound     ; cannot drop onto a push block

                  ld    a,(hl)
                  and   7
                  cp    7
                  jr    z,Man_No_Carry_Drop_Sound     ; block already at max height

                  ld    a,(Player+OFS_HEIGHT)
                  cp    1
                  adc   a,0
                  inc   a
                  ld    c,a
                  call  Can_Move_Here     ; stores new posn+height if can move here
                  jr    nc,Man_No_Carry_Drop_Sound

                  ld    a,(hl)
                  and   248
                  ld    c,a
                  ld    a,(hl)
                  inc   a
                  and   7
                  or    c
                  ld    (hl),a

                  ld    hl,20480+(32*5)+21
                  ld    de,Space32.Gfx
                  call  Draw_16x16

                  xor   a
                  ld    (Carry_Block),a

                  ld    hl,600
                  ld    de,47
                  call  Beep
                  ret

Man_No_Carry_Drop_Sound:
                  ld    hl,1400
                  ld    de,23
                  call  Beep
                  ld    hl,1900
                  ld    de,23
                  call  Beep
                  ret


Got_Hydrogen_Message:
                  ld    hl,20480+(32*5)+24
                  ld    de,Hydrogen_Gfx
                  call  Draw_16x16

                  ld    de,outofcity.txt
                  call  WriteText

                  ld    hl,450
                  ld    de,2000
                  call  Beep

                  call  SetPlayAreaAttrs

                  ld    a,1
                  ld    (Carry_Hydrogen),a
                  ret

outofcity.txt:    db    C_PAPER,2,C_INK,7,C_SINGLE
                  db    C_AT,7,4, "                        "
                  db    C_AT,8,4, " now get this hydrogen  "
                  db    C_AT,9,4, " bomb out of the city!  "
                  db    C_AT,10,4,"                        "
                  db    0

Got_Time:         ld    de,timebonus.txt
                  call  WriteText

gt_releasespace:  ld    a,#7f
                  in    a,(254)
                  rra
                  jr    nc,gt_releasespace

                  ld    ix,22528+(9*32)+6
                  ld    l,0

gt_waitchoice:    ld    a,l
                  add   a,a
                  ld    b,a
                  add   a,a
                  add   a,b   ; A = L * 6

                  ld    (gt_mod1+2),a
                  inc   a
                  ld    (gt_mod2+2),a
                  add   a,31
                  ld    (gt_mod3+2),a
                  inc   a
                  ld    (gt_mod4+2),a

                  ld    c,%01110001
                  call  gt_setattrs

                  push  hl
                  push  ix
                  ld    hl,450
                  ld    de,20
                  call  Beep
                  pop   ix
                  pop   hl
                  ld    b,3
                  call  gt_wait_frames

                  jr    nc,gt_gotchoice   ; space was pressed

                  ld    c,%00010111
                  call  gt_setattrs

                  ld    a,l
                  inc   a
                  and   3
                  ld    l,a
                  jr    gt_waitchoice

gt_gotchoice:     inc   l
                  ld    a,2
                  dec   l
                  jr    z,gt_addtime      ; +2
                  dec   l
                  jr    z,gt_noaddedtime  ; +0
                  ld    a,5
                  dec   l
                  jr    z,gt_addtime      ; +5

                  ld    a,2               ; -2
                  call  Sub_Time
                  ld    bc,#f040
                  call  WhiteNoise
                  ld    bc,#f060
                  call  WhiteNoise
                  jr    gt_exit


gt_noaddedtime:   ld    hl,1350
                  ld    de,80
                  call  Beep
                  jr    gt_exit

gt_addtime:       push  af
                  call  Add_Time
                  pop   af
                  ld    b,a
gt_addping:       push  bc
                  ld    hl,400
                  ld    de,80
                  call  Beep
                  ld    b,1
                  call  wait_frames
                  pop   bc
                  djnz  gt_addping

gt_exit:          ld    b,75
                  call  wait_frames

                  call  SetPlayAreaAttrs
                  ret

gt_setattrs:
gt_mod1:          ld    (ix+0),c
gt_mod2:          ld    (ix+1),c
gt_mod3:          ld    (ix+32),c
gt_mod4:          ld    (ix+33),c
                  ret

gt_wait_frames:   push  hl
gt_wait_floop:    ld    hl,1290
gt_wait_f1:       ld    a,#7f
                  in    a,(254)
                  rra
                  jr    nc,gt_wait_exit   ; no carry = space down
                  dec   hl
                  ld    a,h
                  or    l
                  jr    nz,gt_wait_f1
                  djnz  gt_wait_floop
                  scf                     ; carry = space up
gt_wait_exit:     pop   hl
                  ret

wait_frames:      push  hl
wait_floop:       ld    hl,2728
wait_f1:          dec   hl
                  ld    a,h
                  or    l
                  jr    nz,wait_f1
                  djnz  wait_floop
                  pop   hl
                  ret


timebonus.txt:    db    C_PAPER,2,C_INK,7,C_SINGLE
                  db    C_AT,6,2,  "                            "
                  db    C_AT,7,2,  " press space for time bonus "
                  db    C_AT,8,2,  "                            "
                  db    C_DOUBLE
                  db    C_AT,9,2,  "    +2    +0    +5    -2    "
                  db    C_SINGLE
                  db    C_AT,11,2, "                            "
                  db    0


MAN_GO_LEFT       equ   #2d
MAN_GO_RIGHT      equ   #2c
MAN_GO_UP         equ   #25
MAN_GO_DOWN       equ   #24


Man_Up:           ld    hl,(Player+OFS_X)
                  ld    c,(iy+OFS_HEIGHT)
                  ld    a,(manup_op)
      manup_op:   dec   h
                  jr    Can_Move_Here

Man_Down:         ld    hl,(Player+OFS_X)
                  ld    c,(iy+OFS_HEIGHT)
                  ld    a,(mandown_op)
      mandown_op: inc   h
                  jr    Can_Move_Here

Man_Left:         ld    hl,(Player+OFS_X)
                  ld    c,(iy+OFS_HEIGHT)
                  ld    a,(manleft_op)
      manleft_op: dec   l
                  jr    Can_Move_Here

Man_Right:        ld    hl,(Player+OFS_X)
                  ld    c,(iy+OFS_HEIGHT)
                  ld    a,(manright_op)
      manright_op:inc   l
                  jr    Can_Move_Here


; bits per block in map
; 0-2 = height of this block (0 = no block)
; 3 = set if not solid (only block at height specified)
; 4 = reserved
; 5 = object to collect
; 6+7 = code for anything also in this block location

; HL = map location to test
; C  = Height

Can_Move_Here:    ld    (push_offset1),a

                  ld    a,h
                  cp    high MAP
                  ccf
                  ret   nc
                  ld    a,l
                  cp    #e9
                  ret   nc

                  call  Test_Move_Here
                  ret   nc
                  inc   c                 ; and test for height above (man is 2 chars high)
                  call  Test_Move_Here
                  dec   c
                  ret   nc

                  ld    (Player+OFS_X),hl
                  ld    (iy+OFS_HEIGHT),c

                  scf
                  ret

; returns with Carry set if we can move to this location
Test_Move_Here:
                  ld    a,(hl)
                  bit   5,a
                  jr    nz,test_move_object

                  bit   4,a
                  jr    nz,test_special_block

Test_Move_Normal: ld    a,(hl)
                  and   7
                  scf
                  ret   z           ; no block here; C set

                  cp    c
                  ret   z           ; block at this height; C clear

                  ret   c           ; higher than the block height; C set

                  bit   3,(hl)      ; test for solid column of blocks
                  ret   z           ; block is solid, C clear

                  scf
                  ret

test_move_object: cp    OBJECT_HYDROGEN
                  jr    z,test_got_hydrogen

                  scf
                  ret

test_got_hydrogen:
                  ld    a,(Carry_Hydrogen)
                  cp    1
                  ret

test_special_block:
                  and   %11110000
                  cp    SPECIAL_PUSH
                  jr    z,Pushable_Block
                  or    a
                  ret

Pushable_Block:   ld    a,(testing_man_drop)
                  or    a
                  jr    nz,Test_Move_Normal

                  ld    a,#7f
                  in    a,(254)
                  rra
                  jr    nc,Test_Move_Normal

                  ld    a,(hl)
                  and   7
                  cp    c
                  jr    c,Test_Move_Normal      ; can't push a block below us

                  ld    ixl,0
                  push  hl

  push_loop:      push  hl
                  ld    b,(hl)
  push_offset1:   dec   h
                  ld    a,(hl)
                  or    a
                  jr    z,push_this_block

                  and   %11110000
                  cp    SPECIAL_PUSH
                  jr    nz,push_exit

                  inc   ixl
                  inc   sp
                  inc   sp
                  jr    push_loop


push_exit:        pop   hl
                  pop   hl
                  or    a     ; can't move here
                  ret

push_this_block:  ld    (hl),b

                  push  bc
                  push  de
                  push  ix
                  ld    hl,975
                  ld    de,40
                  call  Beep
                  ld    hl,275
                  ld    de,10
                  call  Beep
                  pop   ix
                  pop   de
                  pop   bc

                  pop   hl
                  ld    (hl),0

                  pop   hl
                  ld    a,ixl
                  cp    1
;                  scf
                  ret

testing_man_drop: db    0

BBpixelup         macro
                  LOCAL exitpixup

                  ld    a,e
                  sub   32
                  ld    e,a
                  jp    nc,exitpixup

                  dec   d
                  ld    a,d
                  cp    high BACKBUFFER   ; C set = out of backbuffer memory
exitpixup:
                  endm

SUBDE             macro subarg
                  LOCAL exit_subde

                  ld    a,e
                  sub   subarg
                  ld    e,a
                  jp    nc,exit_subde

                  dec   d
exit_subde:
                  endm

ADDHL             macro addarg
                  LOCAL exit_addhl

                  ld    a,l
                  add   a,addarg
                  ld    l,a
                  jp    nc,exit_addhl

                  inc   h
exit_addhl:
                  endm

SUBHL             macro subarg
                  LOCAL exit_subhl

                  ld    a,l
                  sub   subarg
                  ld    l,a
                  jp    nc,exit_subhl

                  dec   h
exit_subhl:
                  endm

BBcharup          macro
                  dec   d
                  ld    a,d
                  cp    high BACKBUFFER   ; C set = out of backbuffer memory
                  endm


FACING_LEFT       equ   0
FACING_UP         equ   1
FACING_RIGHT      equ   2
FACING_DOWN       equ   3
view_dir:         db    0
old_view_dir:     db    0

RIGHT_IN_MAP      equ   #1c   ; inc e
LEFT_IN_MAP       equ   #1d   ; dec e
DOWN_IN_MAP       equ   #14   ; inc d
UP_IN_MAP         equ   #15   ; dec d

ViewDir.txt:      db    C_PAPER,1,C_INK,6
                  db    C_DOUBLE
                  db    C_AT,19,28
viewdirchar:      db    " "
                  db    C_SINGLE
                  db    0

Alter_View:       ld    a,#fe       ; V cycles viewing angles
                  in    a,(254)
                  and   16
                  ret   nz

                  ld    a,(view_dir)
                  inc   a
                  and   3
                  call  Set_View

Draw_View_Dir:    ld    de,ViewDir.txt
                  jp    WriteText

Set_View:         ld    (view_dir),a
                  or    a
                  jr    z,Set_Facing_Left
                  dec   a
                  jp    z,Set_Facing_Up
                  dec   a
                  jr    z,Set_Facing_Right
                  jp    Set_Facing_Down


Set_Facing_Left:  ld    a,FACING_LEFT
                  ld    (view_dir),a

                  ld    a,"s"
                  ld    (viewdirchar),a

                  ld    a,RIGHT_IN_MAP
                  ld    (viewop1_1),a
                  ld    (viewop1_2),a
                  ld    (viewop1_3),a
                  ld    (viewop1_4),a

                  ld    a,DOWN_IN_MAP
                  ld    (viewop2_1),a
                  ld    (viewop2_2),a

                  ld    a,UP_IN_MAP
                  ld    (viewop_4),a

                  ld    a,MAN_GO_RIGHT
                  ld    (manleft_op),a
                  ld    a,MAN_GO_LEFT
                  ld    (manright_op),a
                  ld    a,MAN_GO_DOWN
                  ld    (manup_op),a
                  ld    a,MAN_GO_UP
                  ld    (mandown_op),a

                  ld    hl,#FE10
                  ld    (diff+1),hl
                  ret

Set_Facing_Right: ld    a,FACING_RIGHT
                  ld    (view_dir),a

                  ld    a,"n"
                  ld    (viewdirchar),a

                  ld    a,LEFT_IN_MAP
                  ld    (viewop1_1),a
                  ld    (viewop1_2),a
                  ld    (viewop1_3),a
                  ld    (viewop1_4),a

                  ld    a,UP_IN_MAP
                  ld    (viewop2_1),a
                  ld    (viewop2_2),a

                  ld    a,DOWN_IN_MAP
                  ld    (viewop_4),a

                  ld    a,MAN_GO_LEFT
                  ld    (manleft_op),a
                  ld    a,MAN_GO_RIGHT
                  ld    (manright_op),a
                  ld    a,MAN_GO_UP
                  ld    (manup_op),a
                  ld    a,MAN_GO_DOWN
                  ld    (mandown_op),a

                  ld    hl,#02F0
                  ld    (diff+1),hl
                  ret

Set_Facing_Up:    ld    a,FACING_UP
                  ld    (view_dir),a

                  ld    a,"w"
                  ld    (viewdirchar),a

                  ld    a,DOWN_IN_MAP
                  ld    (viewop1_1),a
                  ld    (viewop1_2),a
                  ld    (viewop1_3),a
                  ld    (viewop1_4),a

                  ld    a,LEFT_IN_MAP
                  ld    (viewop2_1),a
                  ld    (viewop2_2),a

                  ld    a,RIGHT_IN_MAP
                  ld    (viewop_4),a

                  ld    a,MAN_GO_DOWN
                  ld    (manleft_op),a
                  ld    a,MAN_GO_UP
                  ld    (manright_op),a
                  ld    a,MAN_GO_LEFT
                  ld    (manup_op),a
                  ld    a,MAN_GO_RIGHT
                  ld    (mandown_op),a

                  ld    hl,#1002
                  ld    (diff+1),hl
                  ret

Set_Facing_Down:  ld    a,FACING_DOWN
                  ld    (view_dir),a

                  ld    a,"e"
                  ld    (viewdirchar),a

                  ld    a,UP_IN_MAP
                  ld    (viewop1_1),a
                  ld    (viewop1_2),a
                  ld    (viewop1_3),a
                  ld    (viewop1_4),a

                  ld    a,RIGHT_IN_MAP
                  ld    (viewop2_1),a
                  ld    (viewop2_2),a

                  ld    a,LEFT_IN_MAP
                  ld    (viewop_4),a

                  ld    a,MAN_GO_UP
                  ld    (manleft_op),a
                  ld    a,MAN_GO_DOWN
                  ld    (manright_op),a
                  ld    a,MAN_GO_RIGHT
                  ld    (manup_op),a
                  ld    a,MAN_GO_LEFT
                  ld    (mandown_op),a

                  ld    hl,#F0FE
                  ld    (diff+1),hl
                  ret


DOBLOCK           macro
                  LOCAL notvalidblock
                  LOCAL notcollectobject

                  ld    a,(de)
                  and   192+32+7          ; block height > 0 or anything else at this block location?
                  jr    z,notvalidblock

                  and   7
                  cp    c
                  jr    c,notvalidblock

                  ld    a,(de)

                  bit   5,a               ; object to collect?
                  jr    z,notcollectobject

                  call  DrawCollectObject
                  jr    notvalidblock

notcollectobject: ld    iy,No_Object
                  and   192
                  call  nz,Find_Object_Struct

                  xor   a
                  ld    (object_drawn),a

                  ld    a,(de)
                  ld    (object_baseaddr),hl
                  call  DrawBlock
notvalidblock:
                  endm

DrawScreen:       ld    a,%00001110       ; default colour for map display area border
                  ld    (displaybordercol),a

                  ld    hl,BACKBUFFER+30

                  ld    b,23  ;20
                  ld    c,0

dsloop1:          push  bc
                  push  de
                  push  hl

                  ld    b,15
blockrowloop1:    ld    a,d
                  cp    high MAP
                  jr    c,skiprow1
                  DOBLOCK
      skiprow1:   dec   l
                  dec   l
      viewop1_1:  dec   e
      viewop2_1:  dec   d
                  djnz  blockrowloop1

                  pop   hl
                  pop   de
                  push  de
                  push  hl

      viewop1_2:  dec   e

                  ADDHL 127

                  ld    b,14
blockrowloop2:    ld    a,d
                  cp    high MAP
                  jr    c,skiprow2
                  DOBLOCK
      skiprow2:   dec   l
                  dec   l
      viewop1_3:  dec   e
      viewop2_2:  dec   d
                  djnz  blockrowloop2

                  pop   hl
                  pop   de
                  pop   bc

ds_next_column:   inc   h
      viewop1_4:  dec   e
      viewop_4:   inc   d

                  ld    a,b
                  cp    8
                  jr    nc,ds_next_row
                  inc   c

ds_next_row:      dec   b
                  jp    nz,dsloop1


                  ld    a,(on_title_screen)
                  or    a
                  ret   nz

                ; if we're carrying hydrogen then use default colour for border area
                  ld    a,(Carry_Hydrogen)
                  or    a
                  ld    a,%00001110       ; default colour for map display area border; this would have changed if another hydrogen bomb was on-screen
                  jr    nz,mapareaborderchk

                  ld    a,(displaybordercol)
mapareaborderchk: ld    hl,22529
                  cp    (hl)
                  ret   z

                ; else continue into 'SetMapAreaBorderCol' to set new map area border colour

SetMapAreaBorderCol:
                  ld    hl,22529
                  ld    de,22530
                  ld    bc,29
                  ld    (hl),a
                  ldir

                  ld    hl,22529+32
                  ld    de,29
                  ld    b,16
SMABC_1:          ld    (hl),a
                  add   hl,de
                  ld    (hl),a
                  inc   hl
                  inc   hl
                  inc   hl
                  djnz  SMABC_1

                  ld    d,h
                  ld    e,l
                  inc   e
                  ld    bc,29
                  ld    (hl),a
                  ldir
                  ret


; DE = map address; requires preserving
; HL = requires preserving
; B  = requires preserving
Find_Object_Struct:
                  push  bc
                  push  hl
                  ld    iy,Player
                  ld    bc,OFS_SIZE
                  ld    a,NUM_ROBOTS+1    ; robots + player structures
FOS_1:            ld    l,(iy+OFS_X)
                  ld    h,(iy+OFS_Y)
                  or    a
                  sbc   hl,de
                  jr    z,FOS_Exit
                  add   iy,bc
                  dec   a
                  jp    nz,FOS_1

                  ld    iy,No_Object

FOS_Exit:         pop   hl
                  pop   bc
                  ret

object_baseaddr:  dw    0
object_drawn:     db    0

DrawBlock:
                  push  bc
                  push  de
                  push  hl

                  ld    b,a   ; block type to B
                  ex    de,hl ; DE = display, HL = map

                  and   7     ; height of block
                  jp    z,exitdrawblock

                  cp    1
                  jp    z,drawsingleblock

                  bit   3,b   ; solid?
                  jr    z,drawsolidblocks

                  dec   a     ; we skip height-1 chars for non-solid blocks, then draw the block top

                  cp    (iy+OFS_HEIGHT)
                  call  nc,Draw_Object

moveblockheight:  ex    af,af'
                  BBcharup
                  jp    c,exitdrawblock
                  ex    af,af'
                  dec   a
                  jr    nz,moveblockheight

                  jp    drawsingleblock

drawsolidblocks:  ex    af,af'

drawbaseblock:    ld    hl,baseblock+15

                  ld    a,(de)
                  and   %00111111
                  or    (hl)
                  ld    (de),a
                  dec   l
                  dec   e
                  ld    a,(de)
                  and   %11111100
                  or    (hl)
                  ld    (de),a
                  dec   l
                  SUBDE 31

                  ld    a,(de)
                  and   %00001111
                  or    (hl)
                  ld    (de),a
                  dec   l
                  dec   e
                  ld    a,(de)
                  and   %11110000
                  or    (hl)
                  ld    (de),a
                  dec   l
                  SUBDE 31

                  ld    a,(de)
                  and   %00000011
                  or    (hl)
                  ld    (de),a
                  dec   l
                  dec   e
                  ld    a,(de)
                  and   %11000000
                  or    (hl)
                  ld    (de),a
                  dec   l
                  SUBDE 31

                  REPT  5
                  ldd
                  ldd
                  SUBDE 30
                  ENDM

                  ld    a,d
                  cp    high BACKBUFFER
                  jp    c,exitdrawblock

                  ; branch to draw the top block if no middle blocks for this column
                  ex    af,af'
                  dec   a
                  jr    z,drawtopblock

DrawMidBlockDuo   macro
                  ld    (hl),d
                  dec   l
                  ld    (hl),e
                  add   hl,bc
                  ld    (hl),%10111011
                  dec   l
                  ld    (hl),%01010101
                  add   hl,bc
                  endm

                  ; enter a loop drawing any middle block chars of this column

                  ld    hl,%1110111010101010
                  ld    bc,-31

drawmidblock:     ex    af,af'
                  ex    de,hl

                  DrawMidBlockDuo
                  DrawMidBlockDuo
                  DrawMidBlockDuo
                  DrawMidBlockDuo

                  ex    de,hl
                  ld    a,d
                  cp    high BACKBUFFER
                  jp    c,exitdrawblock

                  ex    af,af'
                  dec   a
                  jr    nz,drawmidblock

drawtopblock:     ld    hl,singleblock+15

                  REPT  5
                  ldd
                  ldd
                  SUBDE 30
                  ENDM

                  ld    a,(de)
                  and   %00000011
                  or    (hl)
                  ld    (de),a
                  dec   l
                  dec   e
                  ld    a,(de)
                  and   %11000000
                  or    (hl)
                  ld    (de),a
                  dec   l
                  SUBDE 31

                  ld    a,(de)
                  and   %00001111
                  or    (hl)
                  ld    (de),a
                  dec   l
                  dec   e
                  ld    a,(de)
                  and   %11110000
                  or    (hl)
                  ld    (de),a
                  dec   l
                  SUBDE 31

                  ld    a,(de)
                  and   %00111111
                  or    (hl)
                  ld    (de),a
                  dec   l
                  dec   e
                  ld    a,(de)
                  and   %11111100
                  or    (hl)
                  ld    (de),a
                  dec   l
                  SUBDE 31

                  jp    exitdrawblock

drawsingleblock:  ld    hl,singleblock+31

                  ld    a,(de)
                  and   %00111111
                  or    (hl)
                  ld    (de),a
                  dec   l
                  dec   e
                  ld    a,(de)
                  and   %11111100
                  or    (hl)
                  ld    (de),a
                  dec   l
                  SUBDE 31

                  ld    a,(de)
                  and   %00001111
                  or    (hl)
                  ld    (de),a
                  dec   l
                  dec   e
                  ld    a,(de)
                  and   %11110000
                  or    (hl)
                  ld    (de),a
                  dec   l
                  SUBDE 31

                  ld    a,(de)
                  and   %00000011
                  or    (hl)
                  ld    (de),a
                  dec   l
                  dec   e
                  ld    a,(de)
                  and   %11000000
                  or    (hl)
                  ld    (de),a
                  dec   l
                  SUBDE 31

                  REPT  5
                  ldd
                  ldd
                  SUBDE 30
                  ENDM

                  ld    a,d
                  cp    high BACKBUFFER
                  jp    c,exitdrawblock

                  REPT  5
                  ldd
                  ldd
                  SUBDE 30
                  ENDM

                  ld    a,(de)
                  and   %00000011
                  or    (hl)
                  ld    (de),a
                  dec   l
                  dec   e
                  ld    a,(de)
                  and   %11000000
                  or    (hl)
                  ld    (de),a
                  dec   l
                  SUBDE 31

                  ld    a,(de)
                  and   %00001111
                  or    (hl)
                  ld    (de),a
                  dec   l
                  dec   e
                  ld    a,(de)
                  and   %11110000
                  or    (hl)
                  ld    (de),a
                  dec   l
                  SUBDE 31

                  ld    a,(de)
                  and   %00111111
                  or    (hl)
                  ld    (de),a
                  dec   l
                  dec   e
                  ld    a,(de)
                  and   %11111100
                  or    (hl)
                  ld    (de),a

exitdrawblock:    call  Draw_Object

                  pop   hl
                  pop   de
                  pop   bc
                  ret



; needs to preserve ALL registers
; HL = display address
DrawCollectObject:
                  push  af
                  push  bc
                  push  de
                  push  hl

                  ex    de,hl

                  and   %00100011
                  cp    OBJECT_HYDROGEN
                  jr    z,DrawHydrogenObject

                  cp    OBJECT_ENERGY
                  jr    z,DrawEnergyObject

                  cp    OBJECT_TIME
                  jr    z,DrawTimeObject

                  ret

DrawEnergyObject: ld    hl,Object_Gfx+15
                  ld    bc,Object_Mask+15
                  ld    a,8
                  jr    Draw_CollObj_1

DrawTimeObject:   ld    hl,Time_Gfx+31
                  ld    bc,Time_Mask+31
                  ld    a,16
                  jr    Draw_CollObj_1

DrawHydrogenObject:
                  ld    hl,Hydrogen_Gfx+31
                  ld    bc,Hydrogen_Mask+31
                  ld    a,%00001010
                  ld    (displaybordercol),a    ; main map display area border colour when bomb is on-screen
                  ld    a,16

Draw_CollObj_1:   ex    af,af'
                  ld    a,(bc)
                  ex    de,hl
                  and   (hl)
                  ex    de,hl
                  or    (hl)
                  ld    (de),a
                  dec   hl
                  dec   bc
                  dec   e

                  ld    a,(bc)
                  ex    de,hl
                  and   (hl)
                  ex    de,hl
                  or    (hl)
                  ld    (de),a
                  dec   hl
                  dec   bc
                  inc   e

                  BBpixelup
                  jr    c,Draw_CollObj_Exit_1

                  ex    af,af'
                  dec   a
                  jp    nz,Draw_CollObj_1

Draw_CollObj_Exit_1:
                  pop   hl
                  pop   de
                  pop   bc
                  pop   af
                  ret

Time_Gfx:
                  db    %00000000, %00000000
                  db    %00000011, %11110000
                  db    %00001100, %01101000
                  db    %00010001, %00010100
                  db    %00101001, %00101110
                  db    %00100001, %00001010
                  db    %01000001, %00000101
                  db    %01010001, %00010111
                  db    %01000010, %00000101
                  db    %00100100, %00001010
                  db    %00101000, %00101110
                  db    %00010001, %00010100
                  db    %00001100, %01111000
                  db    %00000011, %10100000
                  db    %00000111, %11010000
                  db    %00001111, %11111000

Time_Mask:
                  db    %11111111, %11111111
                  db    %11111100, %00001111
                  db    %11110000, %00000111
                  db    %11100000, %00000011
                  db    %11000000, %00000001
                  db    %11000000, %00000001
                  db    %10000000, %00000000
                  db    %10000000, %00000000
                  db    %10000000, %00000000
                  db    %11000000, %00000001
                  db    %11000000, %00000001
                  db    %11100000, %00000011
                  db    %11110000, %00000111
                  db    %11111100, %00011111
                  db    %11111000, %00001111
                  db    %11110000, %00000111

Object_Gfx:       GFX    %0000001111000000
                  GFX    %0000001111000000
                  GFX    %0000110000110000
                  GFX    %0000111111110000
                  GFX    %0000101111010000
                  GFX    %0000111001110000
                  GFX    %0000111111110000
                  GFX    %0000001111000000

Object_Mask:      GFX    %1111100000011111
                  GFX    %1111100000011111
                  GFX    %1110000000000111
                  GFX    %1110000000000111
                  GFX    %1110000000000111
                  GFX    %1110000000000111
                  GFX    %1110000000000111
                  GFX    %1111100000011111

Hydrogen_Gfx:     GFX      %0000000000000000
                  GFX      %0000000001000000
                  GFX      %0000000010000000
                  GFX      %0000000100000000
                  GFX      %0000000110000000
                  GFX      %0000000110000000
                  GFX      %0000011111100000
                  GFX      %0000111111110000
                  GFX      %0001110001111000
                  GFX      %0001110110111000
                  GFX      %0001110001111000
                  GFX      %0001110110111000
                  GFX      %0001110001111000
                  GFX      %0000111111110000
                  GFX      %0000011111100000
                  GFX      %0000001111000000


Hydrogen_Mask:    GFX      %1111111111111111
                  GFX      %1111111100011111
                  GFX      %1111111000111111
                  GFX      %1111110001111111
                  GFX      %1111110000111111
                  GFX      %1111110000111111
                  GFX      %1111000000001111
                  GFX      %1110000000000111
                  GFX      %1100000000000011
                  GFX      %1100000000000011
                  GFX      %1100000000000011
                  GFX      %1100000000000011
                  GFX      %1100000000000011
                  GFX      %1110000000000111
                  GFX      %1111000000001111
                  GFX      %1111100000011111


; needs to preserve ALL registers
; IY = object structure to draw (Height = 255 = no object structure)
Draw_Object:
                  push  af
                  ld    a,(object_drawn)
                  or    a
                  jr    nz,Draw_Obj_Exit
                  inc   a
                  ld    (object_drawn),a

                  ld    a,(iy+OFS_HEIGHT)
                  cp    255
                  jr    z,Draw_Obj_Exit         ; 255 = no object

                  push  bc
                  push  de
                  push  hl

                  ld    de,(object_baseaddr)    ; base display addr of this map block
                  or    a
                  jr    z,Prep_Draw_Obj
                  ld    b,a

DObj_getheight:   BBcharup
                  jr    c,Draw_Obj_Exit_1
                  djnz  DObj_getheight

                  ld    bc,128
                  ex    de,hl
                  add   hl,bc
                  ex    de,hl

Prep_Draw_Obj:
                  ld    a,16
                  ld    hl,Man_Gfx+31
                  ld    bc,Man_Mask+31

Draw_Obj_1:       ex    af,af'
                  ld    a,(bc)
                  ex    de,hl
                  and   (hl)
                  ex    de,hl
                  or    (hl)
                  ld    (de),a
                  dec   hl
                  dec   bc
                  dec   e

                  ld    a,(bc)
                  ex    de,hl
                  and   (hl)
                  ex    de,hl
                  or    (hl)
                  ld    (de),a
                  dec   hl
                  dec   bc
                  inc   e

                  BBpixelup
                  jr    c,Draw_Obj_Exit_1

                  ex    af,af'
                  dec   a
                  jp    nz,Draw_Obj_1

Draw_Obj_Exit_1:  pop   hl
                  pop   de
                  pop   bc

Draw_Obj_Exit:    pop   af
                  ret

maneye_frame:     db    0

Anim_ManEye:      ld    a,(maneye_frame)
                  inc   a
                  and   7
                  ld    (maneye_frame),a
                  add   a,a
                  add   a,a
                  ld    e,a
                  ld    d,0
                  ld    hl,Man_Eyes
                  add   hl,de
                  ld    de,EyeBytes
                  ldi
                  ldi
                  ldi
                  ldi
                  ret

Man_Eyes:         GFX      %0000111111110000
                  GFX      %0000111111110000

                  GFX      %0000111111010000
                  GFX      %0000111111010000

                  GFX      %0000111110010000
                  GFX      %0000111110010000

                  GFX      %0000111100110000
                  GFX      %0000111100110000

                  GFX      %0000111001110000
                  GFX      %0000111001110000

                  GFX      %0000110011110000
                  GFX      %0000110011110000

                  GFX      %0000100111110000
                  GFX      %0000100111110000

                  GFX      %0000101111110000
                  GFX      %0000101111110000



Man_Gfx:          GFX      %0000011111100000
                  GFX      %0000111111110000
      EyeBytes:   GFX      %0000111111110000
                  GFX      %0000111111110000
                  GFX      %0000111111110000
                  GFX      %0000001111000000
                  GFX      %0000011001100000
                  GFX      %0000111001110000
                  GFX      %0001101001011000
                  GFX      %0001101111011000
                  GFX      %0000100110010000
                  GFX      %0000001111000000
                  GFX      %0000011111100000
                  GFX      %0000000110000000
                  GFX      %0000001111000000
                  GFX      %0000000110000000

Man_Mask:         GFX      %1111000000001111
                  GFX      %1110000000000111
                  GFX      %1110000000000111
                  GFX      %1110000000000111
                  GFX      %1110000000000111
                  GFX      %1111100000011111
                  GFX      %1111000000001111
                  GFX      %1110000000000111
                  GFX      %1100000000000011
                  GFX      %1100000000000011
                  GFX      %1110000000000111
                  GFX      %1111100000011111
                  GFX      %1111000000001111
                  GFX      %1111110000111111
                  GFX      %1111100000011111
                  GFX      %1111110000111111


ClearScreen:      ld    hl,16384
                  ld    de,16385
                  ld    bc,6143
                  ld    (hl),l
                  ldir
                  inc   hl
                  inc   de
                  ld    bc,767
                  ld    (hl),a
                  ldir
                  ret

ClearBackBuffer:  ld    (oldcbbSP+1),sp

BBorg             defl  128*32+32

                  ld    sp,BACKBUFFER+BBorg
                  ld    hl,0
                  ld    b,128
CBB_1:            ds    16,#e5 ; 16 x PUSH HL
                  djnz  CBB_1

oldcbbSP:         ld    sp,0
                  ret

WhiteNoise:       call  RandomBit
                  rlca
                  rlca
                  rlca
                  rlca
                  or    1
                  out   (254),a

                  ld    a,c
noise1:           dec   a
                  jr    nz,noise1
                  djnz  WhiteNoise
                  ret

LargeFallWobble:  ld    hl,BACKBUFFER+32+2
                  ld    bc,#08FF
LargeFW_1:        push  bc
                  push  hl
                  call  CopyBackBuff1

                  ld    bc,#3040
                  call  WhiteNoise

                  pop   hl
                  pop   bc
                  ld    a,l
                  xor   32
                  ld    l,a
                  ld    a,c
                  neg
                  ld    c,a
                  add   a,l
                  ld    l,a
                  djnz  LargeFW_1

                  xor   a
                  ld    (dofallwobble),a

                  ld    a,30
                  call  Sub_Energy
                  ret

SetCopyBufferStack:

CopyBackBuffer:   ld    hl,BACKBUFFER+32+2
CopyBackBuff1:    ld    (CopyBB_SP+1),sp
                  ld    sp,scrtable+(8*2) ; 16384+32
                  ld    bc,128*28   ; 128 lines * 28 bytes per line
CopyBB_1:         pop   de
                  inc   e
                  inc   e     ; always 2 chars across
                  db    237,160,237,160,237,160,237,160,237,160,237,160,237,160     ; 7 x LDI
                  db    237,160,237,160,237,160,237,160,237,160,237,160,237,160     ; 7 x LDI
                  db    237,160,237,160,237,160,237,160,237,160,237,160,237,160     ; 7 x LDI
                  db    237,160,237,160,237,160,237,160,237,160,237,160,237,160     ; 7 x LDI
                  inc   hl
                  inc   hl
                  inc   hl
                  inc   hl
                  jp    pe,CopyBB_1
CopyBB_SP:        ld    sp,0
                  ret

; uses hl, de, a
RandomBit:        ld    hl,(RandomSeed)
                  ld    a,h
                  rlca
                  ld    d,a
                  rlca
                  rlca
                  ld    e,a

                  ld    a,l
                  xor   d
                  xor   e
                  srl   a
                  rr    h
                  rr    l
                  ld    (RandomSeed),hl
                  ld    a,0
                  adc   a,0
                  ret

                  include     "Robots.asm"
                  include     "MapEffects.asm"
                  include     "MapCodec.asm"
                  include     "Text.asm"
                  include     "Misc.asm"
                  include     "TitleEffects.asm"
                  include     "HUD.asm"

Beep:             ld   a,l
                  srl  l
                  srl  l
                  cpl
                  and  3
                  ld   c,a
                  ld   b,0
                  ld   ix,l977
                  add  ix,bc
                  ld   a,8+1
l977:	            nop
                  nop
                  nop
                  inc  b
                  inc  c
l982:	            dec  c
            	jr   nz,l982

            	ld   c,63
            	dec  b
            	jp   nz,l982

            	xor  16
            	out  (254),a
            	ld   b,h
            	ld   c,a
            	bit  4,a
            	jr   nz,l1010

            	ld   a,d
            	or   e
            	jr   z,l1014

            	ld   a,c
            	ld   c,l
            	dec  de
            	jp   (ix)

l1010:	      ld   c,l
	            inc  c
	            jp   (ix)

l1014:            ret


Add_Energy:       ld    c,a
                  ld    a,(energy)
                  add   a,c
                  cp    MAX_ENERGY
                  jr    c,UpdateEnergy
                  ld    a,MAX_ENERGY
                  jr    UpdateEnergy

Sub_Energy:       ld    c,a
                  ld    a,(energy)
                  sub   c
                  jr    nc,UpdateEnergy

                  ld    a,1
                  ld    (man_dead),a
                  xor   a

UpdateEnergy:     ld    (energy),a
                  jr    UpdateEnergyBar

Time_Tick:        ld    hl,(timeword)
                  ld    a,h
                  or    l
                  jr    z,Time_Tick_1
                  dec   hl
                  ld    (timeword),hl
                  ret

Add_Time:         ld    c,a
                  ld    a,(time)
                  add   a,c
                  cp    MAX_TIME
                  jr    c,UpdateTime
                  ld    a,MAX_TIME
                  jr    UpdateTime

Time_Tick_1:      ld    hl,(timewordinit)
                  ld    (timeword),hl
                  ld    a,1
                ; continue into 'Sub_Time'

Sub_Time:         ld    c,a
                  ld    a,(time)
                  sub   c
                  jr    nc,UpdateTime

                  ld    a,1
                  ld    (man_dead),a
                  xor   a

UpdateTime:       ld    (time),a
                  jr    UpdateTimeBar

SetBarAttrs:      ld    d,h
                  ld    e,l
                  inc   e
                  ld    (hl),%00010111
                  ld    bc,4
                  ldir
                  ld    (hl),%00011111
                  ld    bc,15
                  ldir
                  ret

DrawEnergyBar:    ld    hl,32*19+22528+6
                  call  SetBarAttrs

UpdateEnergyBar:  ld    hl,20480+96+6+512 ; line 19, 2 pixels down, 6 chars across
                  ld    a,(energy)
                  ld    c,a
                  jr    Draw_Bar

DrawTimeBar:      ld    hl,32*20+22528+6
                  call  SetBarAttrs

UpdateTimeBar:    ld    hl,20480+128+6+512 ; line 21, 2 pixels down, 6 chars across
                  ld    a,(time)
                  ld    c,a

Draw_Bar:         ld    e,255
UEB1:             ld    d,e
                  ld    a,c
                  sub   8
                  ld    c,a
                  jr    nc,UEB3

                  add   a,8
                  ld    d,a
                  ld    a,8
                  sub   d
UEB2:             or    a
                  rl    e
                  dec   a
                  jr    nz,UEB2

                  ld    d,a         ; d = a = 0
                  ld    c,255

UEB3:             ld    (hl),e
                  inc   h
                  ld    (hl),e
                  inc   h
                  ld    (hl),e
                  inc   h
                  ld    (hl),e

                  dec   h
                  dec   h
                  dec   h

                  ld    e,d
                  inc   l
                  ld    a,l
                  and   31
                  cp    26
                  jr    nz,UEB1
                  ret

Draw_16x16:       ld    b,16
Dr_16x16_1:       ld    a,(de)
                  ld    (hl),a
                  inc   de
                  inc   l
                  ld    a,(de)
                  ld    (hl),a
                  inc   de
                  dec   l
                  call  pixeldown
                  djnz  Dr_16x16_1
                  ret

Space32.Gfx:      ds    32,0


RandomSeed:       dw    1

dofallwobble:     db    0
energy:           db    0
time:             db    0
timeword:         dw    0
timewordinit:     dw    0
framecount:       db    0
on_title_screen:  db    0

Player:           ds    OFS_SIZE
Robots:           ds    NUM_ROBOTS*OFS_SIZE

No_Object:        ds    OFS_SIZE




